import * as path from "path";
import express from "express";
import cookieParser from "cookie-parser";
import { fs, createDebugger, compatRequire } from "@modern-js/utils";
import finalhandler from "finalhandler";
import { run } from "./context";
import registerRoutes from "./registerRoutes";
const debug = createDebugger("express");
const findAppModule = async (apiDir) => {
  const exts = [
    ".ts",
    ".js"
  ];
  const paths = exts.map((ext) => path.resolve(apiDir, `app${ext}`));
  for (const filename of paths) {
    if (await fs.pathExists(filename)) {
      delete require.cache[filename];
      return [
        compatRequire(filename),
        require(filename)
      ];
    }
  }
  return [];
};
const initMiddlewares = (middleware, app) => {
  middleware.forEach((middlewareItem) => {
    const middlewareFunc = typeof middlewareItem === "string" ? compatRequire(middlewareItem) : middlewareItem;
    app.use(middlewareFunc);
  });
};
const useRun = (app) => {
  app.use((req, res, next) => {
    run({
      req,
      res
    }, next);
  });
};
const initApp = (app) => {
  app.use(cookieParser());
  app.use(express.text());
  app.use(express.urlencoded({
    extended: true
  }));
  app.use(express.json());
  return app;
};
var plugin_default = () => ({
  name: "@modern-js/plugin-express",
  pre: [
    "@modern-js/plugin-bff"
  ],
  post: [
    "@modern-js/plugin-server"
  ],
  setup: (api) => ({
    async prepareApiServer({ pwd, config, render }) {
      var _userConfig_bff;
      let app;
      const appContext = api.useAppContext();
      const apiHandlerInfos = appContext.apiHandlerInfos;
      const apiDirectory = appContext.apiDirectory;
      const apiDir = apiDirectory || path.join(pwd, "./api");
      const mode = appContext.apiMode;
      const userConfig = api.useConfigContext();
      if (mode === "framework") {
        const appModule = await findAppModule(apiDir);
        app = appModule[0];
        const hooks = appModule[1];
        if (!(app === null || app === void 0 ? void 0 : app.use)) {
          app = express();
        }
        initApp(app);
        if (config) {
          const { middleware } = config;
          initMiddlewares(middleware, app);
        }
        useRun(app);
        registerRoutes(app, apiHandlerInfos);
        if (hooks) {
          const { afterLambdaRegisted } = hooks;
          if (afterLambdaRegisted) {
            afterLambdaRegisted(app);
          }
        }
      } else if (mode === "function") {
        app = express();
        initApp(app);
        if (config) {
          const { middleware } = config;
          initMiddlewares(middleware, app);
        }
        useRun(app);
        registerRoutes(app, apiHandlerInfos);
      } else {
        throw new Error(`mode must be function or framework`);
      }
      if (((_userConfig_bff = userConfig.bff) === null || _userConfig_bff === void 0 ? void 0 : _userConfig_bff.enableHandleWeb) && render) {
        app.use(async (req, res, next) => {
          const html = await render(req, res);
          if (html) {
            res.end(html);
          }
          next();
        });
      }
      return (req, res) => new Promise((resolve, reject) => {
        const handler = (err) => {
          if (err) {
            return reject(err);
          }
          return finalhandler(req, res, {})(null);
        };
        res.on("finish", (err) => {
          if (err) {
            return reject(err);
          }
          return resolve();
        });
        return app(req, res, handler);
      });
    },
    prepareWebServer({ config }, next) {
      var _userConfig_server;
      const userConfig = api.useConfigContext();
      if (!(userConfig === null || userConfig === void 0 ? void 0 : (_userConfig_server = userConfig.server) === null || _userConfig_server === void 0 ? void 0 : _userConfig_server.enableFrameworkExt)) {
        return next();
      }
      const app = express();
      initApp(app);
      if (config) {
        const { middleware } = config;
        debug("web middleware", middleware);
        initMiddlewares(middleware, app);
      }
      return (ctx) => new Promise((resolve, reject) => {
        const { source: { req, res } } = ctx;
        const handler = (err) => {
          if (err) {
            return reject(err);
          }
          if (res.headersSent && res.statusCode !== 200) {
            finalhandler(req, res, {})(null);
          }
          return resolve();
        };
        res.on("finish", (err) => {
          if (err) {
            return reject(err);
          }
          return resolve();
        });
        return app(req, res, handler);
      });
    }
  })
});
export {
  plugin_default as default
};
