import "reflect-metadata";
import { httpMethods, isWithMetaHandler, isInputParamsDeciderHandler, HttpMetadata, ResponseMetaType, ValidationError } from "@modern-js/bff-core";
import { isSchemaHandler } from "@modern-js/bff-runtime";
import typeIs from "type-is";
import formidable from "formidable";
const handleResponseMeta = (res, handler) => {
  const responseMeta = Reflect.getMetadata(HttpMetadata.Response, handler);
  if (Array.isArray(responseMeta)) {
    for (const meta of responseMeta) {
      const metaType = meta.type;
      const metaValue = meta.value;
      switch (metaType) {
        case ResponseMetaType.Headers:
          for (const [key, value] of Object.entries(metaValue)) {
            res.append(key, value);
          }
          break;
        case ResponseMetaType.Redirect:
          res.redirect(metaValue);
          break;
        case ResponseMetaType.StatusCode:
          res.status(metaValue);
          break;
        default:
          break;
      }
    }
  }
};
const createRouteHandler = (handler) => {
  const apiHandler = async (req, res, next) => {
    const input = await getInputFromRequest(req);
    if (isWithMetaHandler(handler)) {
      try {
        handleResponseMeta(res, handler);
        if (res.headersSent) {
          return;
        }
        const result = await handler(input);
        if (result && typeof result === "object") {
          return res.json(result);
        }
      } catch (error) {
        if (error instanceof ValidationError) {
          res.status(error.status);
          return res.json({
            message: error.message
          });
        }
        throw error;
      }
    } else if (isSchemaHandler(handler)) {
      const result = await handler(input);
      if (result.type !== "HandleSuccess") {
        if (result.type === "InputValidationError") {
          res.status(400);
        } else {
          res.status(500);
        }
        return res.json(result.message);
      } else {
        res.status(200);
        return res.json(result.value);
      }
    } else if (isInputParamsDeciderHandler(handler)) {
      try {
        var _input_data;
        const args = (input === null || input === void 0 ? void 0 : (_input_data = input.data) === null || _input_data === void 0 ? void 0 : _input_data.args) || [];
        const body = await handler(...args);
        if (typeof body !== "undefined") {
          if (typeof body === "object") {
            return res.json(body);
          }
          return res.send(body);
        }
      } catch (e) {
        return next(e);
      }
    } else {
      const args = Object.values(input.params).concat(input);
      try {
        const body = await handler(...args);
        if (res.headersSent) {
          return await Promise.resolve();
        }
        if (typeof body !== "undefined") {
          return res.json(body);
        }
      } catch (e) {
        return next(e);
      }
    }
  };
  Object.defineProperties(apiHandler, Object.getOwnPropertyDescriptors(handler));
  return apiHandler;
};
const isNormalMethod = (httpMethod) => httpMethods.includes(httpMethod);
const getInputFromRequest = async (request) => {
  const draft = {
    params: request.params,
    query: request.query,
    headers: request.headers,
    cookies: request.headers.cookie
  };
  if (typeIs(request, [
    "application/json"
  ])) {
    draft.data = request.body;
  } else if (typeIs(request, [
    "multipart/form-data"
  ])) {
    draft.formData = await resolveFormData(request);
  } else if (typeIs(request, [
    "application/x-www-form-urlencoded"
  ])) {
    draft.formUrlencoded = request.body;
  } else {
    draft.body = request.body;
  }
  return draft;
};
const resolveFormData = (request) => {
  const form = formidable({
    multiples: true
  });
  return new Promise((resolve, reject) => {
    form.parse(request, (err, fields, files) => {
      if (err) {
        reject(err);
      }
      resolve({
        ...fields,
        ...files
      });
    });
  });
};
export {
  createRouteHandler,
  isNormalMethod
};
